@@ -11,10 +11,9 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkCanvPs.c 1.47 96/04/11 20:23:36
+ * SCCS: @(#) tkCanvPs.c 1.50 96/10/02 15:42:41
  */
 
-#include <stdio.h>
 #include "tkInt.h"
 #include "tkCanvas.h"
 #include "tkPort.h"
@@ -59,7 +58,7 @@ typedef struct TkPostscriptInfo {
     char *fileName;		/* Name of file in which to write Postscript;
 				 * NULL means return Postscript info as
 				 * result. Malloc'ed. */
-    FILE *f;			/* Open file corresponding to fileName. */
+    Tcl_Channel chan;		/* Open channel corresponding to fileName. */
     Tcl_HashTable fontTable;	/* Hash table containing names of all font
 				 * families used in output.  The hash table
 				 * values are not used. */
@@ -153,7 +152,6 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 #if !(defined(__WIN32__) || defined(MAC_TCL))
     struct passwd *pwPtr;
 #endif /* __WIN32__ || MAC_TCL */
-    FILE *f;
     size_t length;
     int deltaX = 0, deltaY = 0;		/* Offset of lower-left corner of
 					 * area to be marked up, measured
@@ -165,7 +163,6 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     Tcl_HashSearch search;
     Tcl_HashEntry *hPtr;
     Tcl_DString buffer;
-    char *libDir;
 
     /*
      *----------------------------------------------------------------
@@ -194,7 +191,7 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     psInfo.colorMode = NULL;
     psInfo.colorLevel = 0;
     psInfo.fileName = NULL;
-    psInfo.f = NULL;
+    psInfo.chan = NULL;
     psInfo.prepass = 0;
     Tcl_InitHashTable(&psInfo.fontTable, TCL_STRING_KEYS);
     result = Tk_ConfigureWidget(canvasPtr->interp, canvasPtr->tkwin,
@@ -299,12 +296,9 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	if (p == NULL) {
 	    goto cleanup;
 	}
-	psInfo.f = fopen(p, "w");
+	psInfo.chan = Tcl_OpenFileChannel(canvasPtr->interp, p, "w", 0);
 	Tcl_DStringFree(&buffer);
-	if (psInfo.f == NULL) {
-	    Tcl_AppendResult(canvasPtr->interp, "couldn't write file \"",
-		    psInfo.fileName, "\": ",
-		    Tcl_PosixError(canvasPtr->interp), (char *) NULL);
+	if (psInfo.chan == NULL) {
 	    goto cleanup;
 	}
     }
@@ -398,42 +392,15 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     Tcl_AppendResult(canvasPtr->interp, "%%EndComments\n\n", (char *) NULL);
 
     /*
-     * Read a standard prolog file from disk and insert it into
+     * Read a standard prolog file in a native way and insert it into
      * the Postscript.
      */
 
-    libDir = Tcl_GetVar(canvasPtr->interp, "tk_library", TCL_GLOBAL_ONLY);
-    if (libDir == NULL) {
-	Tcl_ResetResult(canvasPtr->interp);
-	Tcl_AppendResult(canvasPtr->interp, "couldn't find library directory: ",
-		"tk_library variable doesn't exist", (char *) NULL);
+    if (TkGetNativeProlog(canvasPtr->interp) != TCL_OK) {
 	goto cleanup;
     }
-    sprintf(string, "%.350s/prolog.ps", libDir);
-    p = Tcl_TranslateFileName(canvasPtr->interp, string, &buffer);
-    f = fopen(p, "r");
-    Tcl_DStringFree(&buffer);
-    if (f == NULL) {
-	Tcl_ResetResult(canvasPtr->interp);
-	Tcl_AppendResult(canvasPtr->interp, "couldn't open prolog file \"",
-		string, "\": ", Tcl_PosixError(canvasPtr->interp),
-		(char *) NULL);
-	goto cleanup;
-    }
-    while (fgets(string, STRING_LENGTH, f) != NULL) {
-	Tcl_AppendResult(canvasPtr->interp, string, (char *) NULL);
-    }
-    if (ferror(f)) {
-	fclose(f);
-	Tcl_ResetResult(canvasPtr->interp);
-	Tcl_AppendResult(canvasPtr->interp, "error reading prolog file \"",
-		string, "\": ",
-		Tcl_PosixError(canvasPtr->interp), (char *) NULL);
-	goto cleanup;
-    }
-    fclose(f);
-    if (psInfo.f != NULL) {
-	fputs(canvasPtr->interp->result, psInfo.f);
+    if (psInfo.chan != NULL) {
+	Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	Tcl_ResetResult(canvasPtr->interp);
     }
 
@@ -479,8 +446,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	    psInfo.x, Tk_CanvasPsY((Tk_Canvas) canvasPtr, (double) psInfo.y2));
     Tcl_AppendResult(canvasPtr->interp, string,
 	" lineto closepath clip newpath\n", (char *) NULL);
-    if (psInfo.f != NULL) {
-	fputs(canvasPtr->interp->result, psInfo.f);
+    if (psInfo.chan != NULL) {
+	Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	Tcl_ResetResult(canvasPtr->interp);
     }
 
@@ -513,8 +480,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 	    goto cleanup;
 	}
 	Tcl_AppendResult(canvasPtr->interp, "grestore\n", (char *) NULL);
-	if (psInfo.f != NULL) {
-	    fputs(canvasPtr->interp->result, psInfo.f);
+	if (psInfo.chan != NULL) {
+	    Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	    Tcl_ResetResult(canvasPtr->interp);
 	}
     }
@@ -528,8 +495,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
 
     Tcl_AppendResult(canvasPtr->interp, "restore showpage\n\n",
 	    "%%Trailer\nend\n%%EOF\n", (char *) NULL);
-    if (psInfo.f != NULL) {
-	fputs(canvasPtr->interp->result, psInfo.f);
+    if (psInfo.chan != NULL) {
+	Tcl_Write(psInfo.chan, canvasPtr->interp->result, -1);
 	Tcl_ResetResult(canvasPtr->interp);
     }
 
@@ -562,8 +529,8 @@ TkCanvPostscriptCmd(canvasPtr, interp, argc, argv)
     if (psInfo.fileName != NULL) {
 	ckfree(psInfo.fileName);
     }
-    if (psInfo.f != NULL) {
-	fclose(psInfo.f);
+    if (psInfo.chan != NULL) {
+	Tcl_Close(canvasPtr->interp, psInfo.chan);
     }
     Tcl_DeleteHashTable(&psInfo.fontTable);
     canvasPtr->psInfoPtr = oldInfoPtr;
@@ -1144,3 +1111,80 @@ GetPostscriptPoints(interp, string, doublePtr)
     *doublePtr = d;
     return TCL_OK;
 }
+
+/*
+ *--------------------------------------------------------------
+ *
+ * TkGetProlog --
+ *
+ *	Locate and load the postscript prolog.
+ *
+ * Results:
+ *	A standard Tcl Result.  If everything is OK the prolog
+ *	will be located in the result string of the interpreter.
+ *
+ * Side effects:
+ *	None.
+ *
+ *--------------------------------------------------------------
+ */
+
+int
+TkGetProlog(interp)
+    Tcl_Interp *interp;		/* Places the prolog in the result. */
+{
+    char *libDir;
+    Tcl_Channel chan;
+    Tcl_DString buffer, buffer2;
+    char *prologPathParts[2];
+    int bufferSize;
+    char *prologBuffer;
+
+    libDir = Tcl_GetVar(interp, "tk_library", TCL_GLOBAL_ONLY);
+    if (libDir == NULL) {
+	Tcl_ResetResult(interp);
+	Tcl_AppendResult(interp, "couldn't find library directory: ",
+		"tk_library variable doesn't exist", (char *) NULL);
+	return TCL_ERROR;
+    }
+    Tcl_TranslateFileName(interp, libDir, &buffer);
+    prologPathParts[0] = buffer.string;
+    prologPathParts[1] = "prolog.ps";
+    Tcl_DStringInit(&buffer2);
+    Tcl_JoinPath(2, prologPathParts, &buffer2);
+    Tcl_DStringFree(&buffer);
+
+    /*
+     * Compute size of file by seeking to the end of the file.  This will
+     * overallocate if we are performing CRLF translation.
+     */
+    
+    chan = Tcl_OpenFileChannel(interp, buffer2.string, "r", 0);
+    if (chan == NULL) {
+	Tcl_DStringFree(&buffer2);
+	return TCL_ERROR;
+    }
+    bufferSize = Tcl_Seek(chan, 0L, SEEK_END);
+    (void) Tcl_Seek(chan, 0L, SEEK_SET);
+    if (bufferSize < 0) {
+	Tcl_AppendResult(interp, "error seeking to end of file \"",
+		buffer2.string, "\":", Tcl_PosixError(interp), (char *) NULL);
+	Tcl_Close(NULL, chan);
+	Tcl_DStringFree(&buffer2);
+	return TCL_ERROR;
+
+    }
+    prologBuffer = (char *) ckalloc((unsigned) bufferSize+1);
+    bufferSize = Tcl_Read(chan, prologBuffer, bufferSize);
+    Tcl_Close(NULL, chan);
+    if (bufferSize < 0) {
+	Tcl_AppendResult(interp, "error reading file \"", buffer2.string, 
+		"\":", Tcl_PosixError(interp), (char *) NULL);
+	Tcl_DStringFree(&buffer2);
+	return TCL_ERROR;
+    }
+    Tcl_DStringFree(&buffer2);
+    prologBuffer[bufferSize] = 0;
+    Tcl_SetResult(interp, prologBuffer, TCL_DYNAMIC);
+    return TCL_OK;
+}
