@@ -3,7 +3,7 @@
 # This file defines the default bindings for Tk text widgets and provides
 # procedures that help in implementing the bindings.
 #
-# SCCS: @(#) text.tcl 1.44 96/04/16 11:42:24
+# SCCS: @(#) text.tcl 1.46 96/08/23 14:07:32
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1996 Sun Microsystems, Inc.
@@ -34,22 +34,6 @@
 #			and auto-scanning.
 #-------------------------------------------------------------------------
 
-# tkTextClipboardKeysyms --
-# This procedure is invoked to identify the keys that correspond to
-# the "copy", "cut", and "paste" functions for the clipboard.
-#
-# Arguments:
-# copy -	Name of the key (keysym name plus modifiers, if any,
-#		such as "Meta-y") used for the copy operation.
-# cut -		Name of the key used for the cut operation.
-# paste -	Name of the key used for the paste operation.
-
-proc tkTextClipboardKeysyms {copy cut paste} {
-    bind Text <$copy> {tk_textCopy %W}
-    bind Text <$cut> {tk_textCut %W}
-    bind Text <$paste> {tk_textPaste %W}
-}
-
 #-------------------------------------------------------------------------
 # The code below creates the default class bindings for entries.
 #-------------------------------------------------------------------------
@@ -257,7 +241,18 @@ bind Text <Control-slash> {
 bind Text <Control-backslash> {
     %W tag remove sel 1.0 end
 }
-tkTextClipboardKeysyms F16 F20 F18
+bind Text <<Cut>> {
+    tk_textCut %W
+}
+bind Text <<Copy>> {
+    tk_textCopy %W
+}
+bind Text <<Paste>> {
+    tk_textPaste %W
+}
+bind Text <<Clear>> {
+    %W delete sel.first sel.last
+}
 bind Text <Insert> {
     catch {tkTextInsert %W [selection get -displayof %W]}
 }
@@ -373,9 +368,6 @@ bind Text <Meta-Delete> {
 	%W delete {insert -1c wordstart} insert
     }
 }
-if !$tk_strictMotif {
-    tkTextClipboardKeysyms Meta-w Control-w Control-y
-}
 
 # A few additional bindings of my own.
 
